package kaka.cakelight;

import java.io.*;
import java.util.Optional;

import static kaka.cakelight.Main.log;

public class FrameGrabber implements Closeable {
    private Configuration config;
    private File file;
    private int bytesPerFrame;
    private InputStream fileStream;

    private FrameGrabber() {
    }

    public static FrameGrabber from(File videoDevice, Configuration config) {
        FrameGrabber fg = new FrameGrabber();
        fg.config = config;
        fg.file = videoDevice;
        fg.bytesPerFrame = config.video.width * config.video.height * config.video.bpp;
        fg.prepare();
        return fg;
    }

    private boolean prepare() {
        try {
            fileStream = new FileInputStream(file);
            return true;
        } catch (FileNotFoundException e) {
            // TODO: handle java.io.FileNotFoundException: /dev/video1 (Permission denied)
            e.printStackTrace();
            return false;
        }
    }

    /**
     * Must be run in the same thread as {@link #prepare}.
     */
    public Optional<Frame> grabFrame() {
        try {
            byte[] data = new byte[bytesPerFrame];
            int count = fileStream.read(data);
            log("# of bytes read = " + count);
            return Optional.of(Frame.of(data, config));
        } catch (IOException e) {
            e.printStackTrace();
        }

        return Optional.empty();
    }

    @Override
    public void close() throws IOException {
        fileStream.close();
    }
}
